/*--------------------------------------------------------------------------

  LINUXAUDIO.C

  Play signal through Linux Sound Blaster Card

  Andrew Simpson, 22 January 1994


  LINUX VERSION

---------------------------------------------------------------------------*/

#define PROGRAM "audio.c"

#include <sys/soundcard.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <malloc.h>
#include <memory.h>
#define TRUE 1
#define FALSE 0

/* same values as myge.c */
#define SPEAKER 1
#define PHONES 2
#define LINE 4

/*-------------------------------------------------------------------------*/

/* Function Prototypes */

int openAudioDevice();
int outputAudioData(short *data, int noSamples, int sampleRate,
		     double gain, unsigned port);
void closeAudioDevice();

/*-------------------------------------------------------------------------*/

/* Global Variables */

static int audiofd=0,mixerfd=0;

/*-------------------------------------------------------------------------*/

int openAudioDevice()

{
  int stereo=0;
  int sampleSize=16;

  /* Don't bother if already open */
  if (audiofd!=0) return(0);

  /* Open audio device */
  if ((audiofd = open("/dev/dsp", O_WRONLY | O_NDELAY))<0)
    {
      fprintf(stderr,"couldn't open the audio device\n");
      return(0);
    }
 
  /* Open audio device */
  if ((mixerfd = open("/dev/mixer", O_WRONLY))<0)
    {
      fprintf(stderr,"couldn't open the mixer device\n");
      return(0);
    }
 
  /* set up sample size and mode */
  if (ioctl(audiofd, SNDCTL_DSP_STEREO, &stereo)<0)
    {
      fprintf(stderr,"couldn't set mono mode\n");
      return(0);
    }

  if (ioctl(audiofd, SNDCTL_DSP_SAMPLESIZE, &sampleSize)<0)
    {
      fprintf(stderr,"couldn't set the sample size\n");
      return(0);
    }

 return(audiofd);
}
	
/*----------------------------------------------------------------------*/

int outputAudioData (short *data, int noSamples, int sampleRate,
		      double gain, unsigned port )

{
  int bufferSize,actualSampleRate;
  int noBytes,bytesToWrite;
  int volume;
  char *dataPtr;

  /* open audio device if needs be */
  if (audiofd==0)
    if (openAudioDevice()==0) 
      return;

  if (mixerfd!=0)
    {
      volume=(int)(gain * 100.);
      ioctl(mixerfd,SOUND_MIXER_VOLUME,&volume);
    }

  /* set up sample rate */
  actualSampleRate=sampleRate;
  if (ioctl(audiofd, SNDCTL_DSP_SPEED, &actualSampleRate)<0)
    {
      fprintf(stderr,"couldn't set the sample rate\n");
      return;
    }

  /* get buffer size */
  ioctl (audiofd,SNDCTL_DSP_GETBLKSIZE,&bufferSize);

  noBytes=noSamples * sizeof(short);
  dataPtr=(char *)data;

  while (noBytes>0)
    {
      if (noBytes>bufferSize)
	bytesToWrite=bufferSize;
      else 
	bytesToWrite=noBytes;

      if (write(audiofd, dataPtr, bytesToWrite) != bytesToWrite)
	fprintf(stderr,"write error to audio device\n");

      noBytes-=bytesToWrite;
      dataPtr+=bytesToWrite;
    }
  ioctl (audiofd,SNDCTL_DSP_SYNC,0);

  return(actualSampleRate);
}

/*----------------------------------------------------------------------*/

void closeAudioDevice()
     
{
  if (audiofd!=0)
    {
      close(audiofd);
      audiofd=0;
    }
}
 
/*----------------------------------------------------------------------*/


